@@ -104,7 +104,7 @@ def _LRClassifier(indata, labels, C, solver, max_iter, n_jobs, **kwargs) -> Logi
     if solver is None:
         solver = 'sag' if no_cells>50000 else 'lbfgs'
     elif solver not in ('liblinear', 'lbfgs', 'newton-cg', 'sag', 'saga'):
-        raise ValueError(f"🛑 Invalid `solver`, should be one of `liblinear`, `lbfgs`, `newton-cg`, `sag`, and `saga`")
+        raise ValueError(f"🛑 Invalid `solver`, should be one of `'liblinear'`, `'lbfgs'`, `'newton-cg'`, `'sag'`, and `'saga'`")
     logger.info(f"🏋️ Training data using logistic regression")
     if (no_cells > 100000) and (indata.shape[1] > 10000):
         logger.warn(f"⚠️ Warning: it may take a long time to train this dataset with {no_cells} cells and {indata.shape[1]} genes, try to downsample cells and/or restrict genes to a subset (e.g., hvgs)")
@@ -160,10 +160,10 @@ def train(X = None,
           #mini-batch
           mini_batch: bool = False, batch_number: int = 100, batch_size: int = 1000, epochs: int = 10, balance_cell_type: bool = False,
           #feature selection
-          feature_selection: bool = False, top_genes: int = 500,
+          feature_selection: bool = False, top_genes: int = 300,
           #description
-          date: str = '', details: str = '', url: str = '',
-          #other SGD param
+          date: str = '', details: str = '', url: str = '', source: str = '', version: str = '',
+          #other param
           **kwargs
          ) -> Model:
     """
@@ -240,19 +240,23 @@ def train(X = None,
         This argument is relevant only if mini-batch SGD training is conducted (`use_SGD = True` and `mini_batch = True`).
         (Default: `False`)
     feature_selection
-        Whether to perform two-pass data training where the first round is used for selecting important features/genes.
-        If `True`, the training time will be approximately doubled.
+        Whether to perform two-pass data training where the first round is used for selecting important features/genes using SGD learning.
+        If `True`, the training time will be longer.
         (Default: `False`)
     top_genes
         The number of top genes selected from each class/cell-type based on their absolute regression coefficients.
         The final feature set is combined across all classes (i.e., union).
-        (Default: 500)
+        (Default: 300)
     date
         Free text of the date of the model. Default to the time when the training is completed.
     details
         Free text of the description of the model.
     url
         Free text of the (possible) download url of the model.
+    source
+        Free text of the source (publication, database, etc.) of the model.
+    version
+        Free text of the version of the model.
     **kwargs
         Other keyword arguments passed to :class:`~sklearn.linear_model.LogisticRegression` (`use_SGD = False`) or :class:`~sklearn.linear_model.SGDClassifier` (`use_SGD = True`).
 
@@ -286,7 +290,7 @@ def train(X = None,
     indata = scaler.fit_transform(indata[:, ~flag] if flag.sum() > 0 else indata)
     indata[indata > 10] = 10
     #classifier
-    if use_SGD:
+    if use_SGD or feature_selection:
         classifier = _SGDClassifier(indata = indata, labels = labels, alpha = alpha, max_iter = max_iter, n_jobs = n_jobs, mini_batch = mini_batch, batch_number = batch_number, batch_size = batch_size, epochs = epochs, balance_cell_type = balance_cell_type, **kwargs)
     else:
         classifier = _LRClassifier(indata = indata, labels = labels, C = C, solver = solver, max_iter = max_iter, n_jobs = n_jobs, **kwargs)
@@ -313,6 +317,6 @@ def train(X = None,
     classifier.features = genes
     if not date:
         date = str(datetime.now())
-    description = {'date': date, 'details': details, 'url': url, 'number_celltypes': len(classifier.classes_)}
+    description = {'date': date, 'details': details, 'url': url, 'source': source, 'version': version, 'number_celltypes': len(classifier.classes_)}
     logger.info(f"✅ Model training done!")
     return Model(classifier, scaler, description)
